require 'scim/query/filter/parser'
require 'test/unit'
require 'yaml'
require 'json'

class TestSpec < Test::Unit::TestCase
  def test_spec
    data = YAML.load $test_spec_data
    parser = SCIM::Query::Filter::Parser.new
    data.each do |test_case|
      query = test_case['query']
      parser.parse query

      got_rpn_json = parser.rpn.to_json
      want_rpn_json = test_case['rpn'].to_json
      assert_equal want_rpn_json, got_rpn_json,
        "Test parse to RPN: '#{query}'"

      got_tree_json = parser.tree.to_json
      want_tree_json = test_case['tree'].to_json
      assert_equal want_tree_json, got_tree_json,
        "Test parse to tree: '#{query}'"
    end
  end
end

# See http://www.simplecloud.info/specs/draft-scim-api-01.html#query-resources
$test_spec_data = <<'...'
- query: ''
  rpn:   []
  tree:  []

- query: userName eq "bjensen"
  rpn:   [username,'"bjensen"',eq]
  tree:  [eq, username,'"bjensen"']

- query: name.familyName co "O'Malley"
  rpn:   [name.familyname, '"O''Malley"', co]
  tree:  [co, name.familyname, '"O''Malley"']

- query: userName sw "J"
  rpn:   [username, '"J"', sw]
  tree:  [sw, username, '"J"']

- query: title pr
  rpn:   [title, pr]
  tree:  [pr, title]

- query: meta.lastModified gt "2011-05-13T04:42:34Z"
  rpn:   [meta.lastmodified, '"2011-05-13T04:42:34Z"', gt]
  tree:  [gt, meta.lastmodified, '"2011-05-13T04:42:34Z"']

- query: meta.lastModified ge "2011-05-13T04:42:34Z"
  rpn:   [meta.lastmodified, '"2011-05-13T04:42:34Z"', ge]
  tree:  [ge, meta.lastmodified, '"2011-05-13T04:42:34Z"']

- query: meta.lastModified lt "2011-05-13T04:42:34Z"
  rpn:   [meta.lastmodified, '"2011-05-13T04:42:34Z"', lt]
  tree:  [lt, meta.lastmodified, '"2011-05-13T04:42:34Z"']

- query: meta.lastModified le "2011-05-13T04:42:34Z"
  rpn:   [meta.lastmodified, '"2011-05-13T04:42:34Z"', le]
  tree:  [le, meta.lastmodified, '"2011-05-13T04:42:34Z"']

- query: title pr and userType eq "Employee"
  rpn:   [title, pr, usertype, '"Employee"', eq, and]
  tree:  [and, [pr, title], [eq, usertype, '"Employee"']]

- query: title pr or userType eq "Intern"
  rpn:   [title, pr, usertype, '"Intern"', eq, or]
  tree:  [or, [pr, title], [eq, usertype, '"Intern"']]

- query: userType eq "Employee" and (emails co "example.com" or emails co "example.org")
  rpn:   [usertype, '"Employee"', eq, emails, '"example.com"', co, emails, '"example.org"', co, or ,and]
  tree:  [and, [eq, usertype, '"Employee"'], [or, [co, emails, '"example.com"'], [co, emails, '"example.org"']]]
...
